@@ -1,19 +1,20 @@
 # # Unity ML-Agents Toolkit
-from typing import Dict, List, Deque, Any
+from typing import List, Deque
 import abc
 
 from collections import deque
 
 from mlagents_envs.logging_util import get_logger
+from mlagents_envs.timers import timed
+from mlagents_envs.base_env import BehaviorSpec
 from mlagents.model_serialization import export_policy_model, SerializationSettings
 from mlagents.trainers.policy.tf_policy import TFPolicy
 from mlagents.trainers.stats import StatsReporter
 from mlagents.trainers.trajectory import Trajectory
 from mlagents.trainers.agent_processor import AgentManagerQueue
-from mlagents.trainers.brain import BrainParameters
 from mlagents.trainers.policy import Policy
-from mlagents.trainers.exception import UnityTrainerException
 from mlagents.trainers.behavior_id_utils import BehaviorIdentifiers
+from mlagents.trainers.settings import TrainerSettings
 
 
 logger = get_logger(__name__)
@@ -25,33 +26,30 @@ class Trainer(abc.ABC):
     def __init__(
         self,
         brain_name: str,
-        trainer_parameters: dict,
+        trainer_settings: TrainerSettings,
         training: bool,
-        run_id: str,
+        artifact_path: str,
         reward_buff_cap: int = 1,
     ):
         """
         Responsible for collecting experiences and training a neural network model.
-        :BrainParameters brain: Brain to be trained.
-        :dict trainer_parameters: The parameters for the trainer (dictionary).
-        :bool training: Whether the trainer is set for training.
-        :str run_id: The identifier of the current run
-        :int reward_buff_cap:
+        :param brain_name: Brain name of brain to be trained.
+        :param trainer_settings: The parameters for the trainer (dictionary).
+        :param training: Whether the trainer is set for training.
+        :param artifact_path: The directory within which to store artifacts from this trainer
+        :param reward_buff_cap:
         """
-        self.param_keys: List[str] = []
         self.brain_name = brain_name
-        self.run_id = run_id
-        self.trainer_parameters = trainer_parameters
-        self.summary_path = trainer_parameters["summary_path"]
-        self._threaded = trainer_parameters.get("threaded", True)
-        self._stats_reporter = StatsReporter(self.summary_path)
+        self.trainer_settings = trainer_settings
+        self._threaded = trainer_settings.threaded
+        self._stats_reporter = StatsReporter(brain_name)
         self.is_training = training
         self._reward_buffer: Deque[float] = deque(maxlen=reward_buff_cap)
         self.policy_queues: List[AgentManagerQueue[Policy]] = []
         self.trajectory_queues: List[AgentManagerQueue[Trajectory]] = []
         self.step: int = 0
-        self.summary_freq = self.trainer_parameters["summary_freq"]
-        self.next_summary_step = self.summary_freq
+        self.artifact_path = artifact_path
+        self.summary_freq = self.trainer_settings.summary_freq
 
     @property
     def stats_reporter(self):
@@ -60,20 +58,12 @@ class Trainer(abc.ABC):
         """
         return self._stats_reporter
 
-    def _check_param_keys(self):
-        for k in self.param_keys:
-            if k not in self.trainer_parameters:
-                raise UnityTrainerException(
-                    "The hyper-parameter {0} could not be found for the {1} trainer of "
-                    "brain {2}.".format(k, self.__class__, self.brain_name)
-                )
-
     @property
-    def parameters(self) -> Dict[str, Any]:
+    def parameters(self) -> TrainerSettings:
         """
         Returns the trainer parameters of the trainer.
         """
-        return self.trainer_parameters
+        return self.trainer_settings
 
     @property
     def get_max_steps(self) -> int:
@@ -81,7 +71,7 @@ class Trainer(abc.ABC):
         Returns the maximum number of steps. Is used to know when the trainer should be stopped.
         :return: The maximum number of steps of the trainer
         """
-        return int(float(self.trainer_parameters["max_steps"]))
+        return self.trainer_settings.max_steps
 
     @property
     def get_step(self) -> int:
@@ -119,6 +109,7 @@ class Trainer(abc.ABC):
         """
         return self._reward_buffer
 
+    @timed
     def save_model(self, name_behavior_id: str) -> None:
         """
         Saves the model
@@ -130,7 +121,7 @@ class Trainer(abc.ABC):
         Exports the model
         """
         policy = self.get_policy(name_behavior_id)
-        settings = SerializationSettings(policy.model_path, policy.brain.brain_name)
+        settings = SerializationSettings(policy.model_path, self.brain_name)
         export_policy_model(settings, policy.graph, policy.sess)
 
     @abc.abstractmethod
@@ -143,7 +134,7 @@ class Trainer(abc.ABC):
 
     @abc.abstractmethod
     def create_policy(
-        self, parsed_behavior_id: BehaviorIdentifiers, brain_parameters: BrainParameters
+        self, parsed_behavior_id: BehaviorIdentifiers, behavior_spec: BehaviorSpec
     ) -> TFPolicy:
         """
         Creates policy
